import 'package:dio/dio.dart';

class HTTPManager {
  Dio _dio = Dio();

  Map<String, CancelToken> _cancelTokenMap = {};

  static HTTPManager? _instance;

  HTTPManager._();


  static _getInstance() {
    if (_instance == null) {
      _instance = HTTPManager._();
    }
    return _instance;
  }

  Dio get dio => _dio;

  static HTTPManager get instance => _getInstance();

  void init(BaseOptions options, {List<Interceptor>? interceptors}) {
    _dio.options = options;
    if (interceptors != null) {
      _dio.interceptors.addAll(interceptors);
    }
  }

  String _prefixCancelKey = "_dio_life_";

  String _subjoinKey(dynamic key) => "${key.toString()}$_prefixCancelKey${DateTime.now().millisecondsSinceEpoch}";

  /// 绑定 CancelToken
  CancelToken addLife(dynamic life) {
    var key = _subjoinKey(life);
    CancelToken cancelToken = CancelToken();
    _cancelTokenMap[key] = cancelToken;
    return cancelToken;
  }

  /// 取消请求
  bool cancelLife(dynamic life) {
    var r = false;
    var keys = List.of(_cancelTokenMap.keys);
    keys.forEach((element) {
      if (element.startsWith("${life.toString()}$_prefixCancelKey")) {
        var cancelToken = _cancelTokenMap[element];
        cancelToken?.cancel();
        _cancelTokenMap.remove(element);
        r = true;
      }
    });
    return r;
  }

  void _removeLife(dynamic life) {
    var keys = List.of(_cancelTokenMap.keys);
    keys.forEach((element) {
      if (element.startsWith("${life.toString()}$_prefixCancelKey")) {
        _cancelTokenMap.remove(element);
      }
    });
  }

  /// 取消当前的全部请求
  void clearLife() {
    _cancelTokenMap.values.forEach((element) => element.cancel());
    _cancelTokenMap.clear();
  }


  Future<Response<T>> getAsync<T>(String path, {Map<String, dynamic>? queryParameters, Options? options, dynamic life, ProgressCallback? onReceiveProgress}) async {
    if (life == null) life = DateTime.now().millisecondsSinceEpoch.toString();
    var cancelToken = addLife(life);
    Response<T> future = await _dio.get<T>(path, queryParameters: queryParameters, options: options, onReceiveProgress: onReceiveProgress, cancelToken: cancelToken);
    _removeLife(life);
    return future;
  }

  Future<Response<T>> postAsync<T>(String path, {data, Options? options, dynamic life, ProgressCallback? onSendProgress, ProgressCallback? onReceiveProgress}) async {
    if (life == null) life = DateTime.now().millisecondsSinceEpoch.toString();
    var cancelToken = addLife(life);
    Response<T> future = await _dio.post(path, data: data, options: options, onSendProgress: onReceiveProgress, onReceiveProgress: onReceiveProgress, cancelToken: cancelToken);
    _removeLife(life);
    return future;
  }
}

mixin DioLife {}

extension MapToFormData on Map<String, dynamic> {
  FormData get toFormData => FormData.fromMap(this);
}
